<?php
/**
 * Plugin Name: Ship-Quik
 * Version: 1.0.0
 * Plugin URI: https://ship-quik.com/
 * Description: Conector for Ship Quik.
 * Requires at least: 4.7
 * Tested up to: 6.0.2
 *
 * Text Domain: ship-quik
 * Domain Path: /lang/
 *
 * @package WordPress
 * @author  Ship Quik
 * @since   1.0.0
 */

if (! defined('ABSPATH') ) {
    exit;
}

define('SHIP_QUIK_PLATFORM', 'woocommerce');

define('SHIP_QUIK_VERSION', '1.0.0');
define('SHIP_QUIK_PLUGIN_PATH', plugin_dir_path(__FILE__));
define('SHIP_QUIK_CLASSES_PATH', SHIP_QUIK_PLUGIN_PATH . 'ShipQuik/');
define('SHIP_QUIK_TEMPLATE_PATH', SHIP_QUIK_PLUGIN_PATH . 'templates/');
define('SHIP_QUIK_TEMPLIST_PATH', SHIP_QUIK_PLUGIN_PATH . 'templist/');

define('CONFIG_DATA', plugin_dir_path(__FILE__) . '/config.json');

function ship_quik_plugin_init()
{
    if (current_user_can('activate_plugins') && !class_exists('WooCommerce') ) {
        add_action('admin_init', 'plugin_deactivate');
        add_action('admin_notices', 'plugin_admin_notice');

        function plugin_deactivate()
        {
            deactivate_plugins(plugin_basename(__FILE__));
        }

        function plugin_admin_notice()
        {
            $dpa_child_plugin = __('Ship-Quik', 'ship-quik');
            $dpa_parent_plugin = __('Woocommerce', 'ship-quik');
            echo '<div class="error"><p>'
                . sprintf(__('%1$s requiere %2$s para funcionar correctamente. Por favor  activa %2$s antes de activar %1$s. Hasta ese momento, Ship-Quik estará desactivado.'), '<strong>' . $dpa_child_plugin . '</strong>', '<strong>' . $dpa_parent_plugin . '</strong>')
                . '</p></div>';
        }

    } else {
        //Load composer
        require SHIP_QUIK_PLUGIN_PATH .'vendor/autoload.php';

        //Load templist
        require SHIP_QUIK_PLUGIN_PATH .'/bootstrap.php';

        // Define PLUGIN_FILE.
        if (! defined('SHIP_QUIK_PLUGIN_FILE') ) {
            define('SHIP_QUIK_PLUGIN_FILE', __FILE__);
        }

        /**
         * Returns the main instance of Ship_Quik to prevent the need to use globals.
         *
         * @since  1.0.0
         * @return object Ship_Quik
         */
        function ship_quik()
        {
            $instance = \ShipQuik\WooCommerce\ShipQuik_Main::instance(__FILE__, SHIP_QUIK_VERSION);

            if (is_admin() && is_null($instance->settings) ) {
                $instance->settings = \ShipQuik\WooCommerce\ShipQuik_Settings::instance($instance);
            }

            $instance->install();

            return $instance;
        }

        ship_quik();
    }
}

add_action('plugins_loaded', 'ship_quik_plugin_init');
