<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class ShipQuik_MultiCurl
{
    private static $_instance = null;

    protected $parent;

    protected $config = null;

    protected $_params = [];

    protected $_is_post = true;

    protected $_verbose = false;

    protected $suppliers = [];

    public function __construct($parent = null, $params = [], $verbose = false, $is_post = true)
    {
        $this->parent  = $parent;
        $this->config  = $parent->config;

        $this->_params  = $params;
        $this->_is_post = $is_post;
        $this->_verbose = $verbose;
    }

    public function getCurtHandle($supplier, $connecttimeout = 800, $timeout = 800, $params = [])
    {
        $curl = curl_init($supplier);
        curl_setopt($curl, CURLOPT_URL, $params['Baseurl']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_NOSIGNAL, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $connecttimeout);
        curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);

        $opciones = [
            'Content-Type: application/json; charset=utf-8',
            'Content-Length: ' . strlen(json_encode($params['Data'])),
            'Expect: '
        ];

        if (WC()->session->get($this->parent->_token . '_auth')) {
            $auth = WC()->session->get($this->parent->_token . '_auth');

            if (isset($auth['token'])) {
                array_push($opciones, 'access-token: ' . $auth['token']);
            }
        }

        if ($this->_is_post) {
            curl_setopt($curl, CURLOPT_POST, $this->_is_post);
            curl_setopt(
                $curl,
                CURLOPT_POSTFIELDS,
                json_encode($params['Data'])
            );
            curl_setopt(
                $curl,
                CURLOPT_HTTPHEADER,
                $opciones
            );
        }

        if ($this->_verbose) {
            curl_setopt($curl, CURLOPT_VERBOSE, true);
        }

        return $curl;
    }

    public function curlCall($connecttimeout = 800, $timeout = 800) {

        $multiCurl  = curl_multi_init();
        $handleList = [];

        foreach ($this->_params as $supplier => $param) {
            $handleList[$supplier] = $this->getCurtHandle($supplier, $connecttimeout, $timeout, $param);
            curl_multi_add_handle($multiCurl, $handleList[$supplier]);
        }

        $running = null;
        do {
            curl_multi_exec($multiCurl, $running);
        } while ($running);

        foreach ($this->_params as $supplier => $param) {
            curl_multi_remove_handle($multiCurl, $handleList[$supplier]);
        }

        curl_multi_close($multiCurl);

        $result = [];

        foreach ($this->_params as $supplier => $param) {
            $result[$supplier] = json_decode(curl_multi_getcontent($handleList[$supplier]), true);
        }

        $resultArray = [];
        foreach ($result as $supplier => $content) {
            if (isset($content['data'])) {
                foreach ($content['data'] as $service) {
                    $resultArray[] = $service;
                }
            }
        }

        return $resultArray;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
