<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \ShipQuik\Shared\ShipQuik_API;
use \ShipQuik\Shared\ShipQuik_Helper;
use \ShipQuik\WooCommerce\ShipQuik_Shipments;

use DateTime;

class ShipQuik_ShippingDua
{
    private static $_instance = null;

    protected $parent;

    protected $boxes;

    protected $destination;

    protected $supplier;

    protected $declaredValue;

    protected $config;

    protected $data;

    public function __construct($parent = null, $boxes = [], $destination = [], $supplier = '', $declaredValue = 0)
    {
        $this->parent = $parent;
        $this->boxes = $boxes;
        $this->destination = $destination;
        $this->supplier = $supplier;
        $this->declaredValue = $declaredValue;

        $this->config = $parent->config;

        $this->data = $this->getData($boxes, $destination, $supplier, $declaredValue);
    }

    private function getData($boxes, $destination, $supplier = '', $declaredValue = 0)
    {
        $supplierData = [];

        $customerAddress = $destination;

        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $supplierData['postalCodeOrigin']       = $senderAddress['PostalCode'];
        $supplierData['postalCodeDestination']  = $customerAddress['postcode'];
        $supplierData['countryCodeOrigin']      = $senderAddress['CountryCode'];
        $supplierData['countryCodeDestination'] = $customerAddress['country'];
        $supplierData['cityOrigin']             = $senderAddress['City'];
        $supplierData['cityDestination']        = $customerAddress['city'];

        $supplierData['pesoEnKG']               = true;

        $supplierData['boxes']                  = $boxes;

        $supplierData['supplier']               = $supplier;
        $supplierData['declaredValue']          = $declaredValue;

        return $supplierData;
    }

    public function send()
    {
        $result = false;

        $params = [];

        $params['Baseurl']           = $this->config->getValueByKey('endpoints')['getDua'];
        $params['Data']              = $this->data;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == "renew_token") {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        if ($result) {
            if (isset($result['data'])) {
                return $result;
            }
        }

        return $result;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
