<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce\FrontEnd;

use \ShipQuik\WooCommerce\ShipQuik_Shipments;
use \ShipQuik\WooCommerce\ShipQuik_ShippingRate;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class ShipQuik_CheckOut
{
    /**
     * Parent object.
     *
     * @var    object
     * @access private
     * @since  1.0.0
     */
    protected $parent = null; 

    /**
     * General configuration.
     *
     * @var    object
     * @access public
     * @since  1.0.0
     */
    protected $config = null;

    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->config = $parent->config;
    }

    // public function after_thankyou($order_id)
    // {
    //     if ($order_id) {

    //         $shiQuikShipments = new ShipQuik_Shipments($this->parent, $order_id);

    //         // TODO: desactivado de momento.
    //         //$order = wc_get_order($order_id);
    //         //$status = $order->get_status('edit');

    //         //if ($this->config->getValueByKey('status_create_shipment') == 'wc-' . $status) {
    //         //    $this->processSendOrder($order_id, false);
    //         //}
    //     }
    // }

    public function order_shipping_to_display_shipped_via($shipped_via, $order) {
        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }
        $supplier = [];

        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == 'Supplier') {
                $supplier = $item['value'];

                break;
            }
        }

        if ($supplier) {
            $shipped_via = '&nbsp;<small class="shipped_via">' . sprintf(__( 'via %s', 'woocommerce' ), $supplier) . '</small>';
        }

        return $shipped_via;
    }

    public function adminErrorNotice()
    {
        echo '<div class="notice notice-warning is-dismissible">' . $this->errorMessage . '</div>';
        $this->errorMessage = '';
    }

    public function processSendOrder($order_id, $ajax = true)
    {
        $order = wc_get_order($order_id);

        $trackingNumber  = $order->get_meta('_shipquik_tracking_number');
        $pickupNumber    = $order->get_meta('_shipquik_pickup_number');
        $pickupDatetime  = $order->get_meta('_shipquik_pickupdatetime');

        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod  = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }

        if (ShipQuik_ShippingRate::isShipQuik($shippingMethod) && !$pickupNumber && !$trackingNumber && $pickupDatetime) {
            $shiQuikShipments = new ShipQuik_Shipments($this->parent, $order->get_id());

            $result = $shiQuikShipments->send();

            if ($result) {
                if (isset($result['code'])) {
                    $response = array(
                        "status"  => $result['code'],
                        "message" => $result['message']
                    );
                    if (!$order->get_meta('_shipquik_label_error')) {
                        $order->update_meta_data('_shipquik_label_error', $response);
                        $order->save();
                    }

                    echo json_encode($response);

                } elseif ($result['status'] == 200) {
                    $fileType = $result['data']['shippingOrder']['labelsMimeType'];

                    if (isset($result['data']['shippingOrder']['trackingNumber'])) {
                        $order->update_meta_data('_shipquik_tracking_number', $result['data']['shippingOrder']['trackingNumber']);
                    }
                    if (isset($result['data']['shippingOrder']['pickupRequestNumber'])) {
                        $order->update_meta_data('_shipquik_pickup_number', $result['data']['shippingOrder']['pickupRequestNumber']);
                    }

                    if (isset($result['data']['shippingOrder']['shippingLabels'])) {
                        $order->update_meta_data('shipquik_label_type', $fileType);

                        foreach ($result['data']['shippingOrder']['shippingLabels'] as $key => $value) {
                            $tmpName = SHIP_QUIK_DOCSPATH . '/' . $order->get_id() . '-' . $key .  ($fileType == 'image/png' ? '.png' : '.pdf');
                            if (!file_exists($tmpName)) {
                                $file = fopen($tmpName, 'x+');
                                fwrite($file, base64_decode($result['data']['shippingOrder']['shippingLabels'][$key]));
                                fclose($file);
                            }
                        }
                    }
                    $order->save_meta_data();

                    if ($ajax) {
                        $response = array(
                            "status"     => 200,
                            "data"       => $fileType
                        );

                        echo json_encode($response);
                    }
                }
            }
        }

        if ($ajax) die();
    }

    public function after_thankyou($order_id)
    {
        if ($order_id) {

            $order = wc_get_order($order_id);

            $shiQuikShipments = new ShipQuik_Shipments($this->parent, $order_id);

            $order->update_meta_data('_shipquik_payment', $shiQuikShipments);
            $order->update_meta_data('_shipquik_payment_status', '0');

            $order->save();
        }
    }

    public function process_ajax_sendOrder()
    {
        if (!isset($_REQUEST['nonce'])) {
            exit("No naughty business please");
        }
    

        $nonce = sanitize_key($_REQUEST['nonce']);
        if (!wp_verify_nonce($nonce, "process_sendorder")) {
            exit("No naughty business please");
        }

        $key = sanitize_text_field($_REQUEST['key']);
        $shiQuikShipments = new ShipQuik_Shipments($this->parent, $key);

        echo esc_attr($this->processSendOrder($key));
    }
}
