<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use \libphonenumber\PhoneNumberUtil;
use \ShipQuik\Shared\ShipQuik_API;
use \ShipQuik\Shared\ShipQuik_ShippingTaxes;
use \ShipQuik\Shared\ShipQuik_Suppliers;
use \ShipQuik\Shared\ShipQuik_Supplier;
use \ShipQuik\Shared\ShipQuik_Configuration;
use \ShipQuik\Shared\ShipQuik_Helper;

class ShipQuik_Shipments
{
    private static $_instance = null;

    protected $parent;

    protected $order_id;

    protected $boxes;

    protected $config;

    public $data;

    protected $PhoneNumberUtil;

    public function __construct($parent = null, $order_id = 0, $boxes = [])
    {
        $this->parent = $parent;
        $this->order_id = $order_id;
        $this->boxes = $boxes;

        $this->config = $parent->config;

        $this->PhoneNumberUtil = PhoneNumberUtil::getInstance();

        $this->data = $this->getData($this->order_id, $this->boxes);
    }

    private function getData($order_id, $boxes)
    {
        $order = wc_get_order($this->order_id);

        $supplierInfo = [];

        $shippingMethods = $order->get_shipping_methods();

        $shippingMethod = [];

        // TODO: We must have only one.
        foreach ($shippingMethods as $key => $value) {
            $shippingMethod = $value;
        }

        $shipment = [];

        $senderAddress = self::getSenderAddress($this->config);

        if (ShipQuik_Configuration::get_option('woocommerce_ship_to_destination') == 'shipping') {
            $customerAddress = $order->get_address('shipping');
        } else {
            $customerAddress = $order->get_address('billing');
        }

        // Shipment Object
        $shipment['shipment']['isKg']  = true;

        if (empty($boxes)) {
            $shipment['shipment']['boxes'] = ShipQuik_ShippingPackage::getBoxes($this->config, $order->get_items());
        } else {
            $shipment['shipment']['boxes'] = $boxes;
        }

        $shipment['shipment']['originAddress'] = [
            'street'      => $senderAddress['Street'],
            'city'        => $senderAddress['City'],
            'countryCode' => $senderAddress['CountryCode'],
            'postalCode'  => $senderAddress['PostalCode'],
            'floor'       => '',
            'door'        => ''
        ];

        $shipment['shipment']['destinationAddress'] = [
            'street'      => $customerAddress['address_1'] . ' ' .$customerAddress['address_2'],
            'city'        => $customerAddress['city'],
            'countryCode' => $customerAddress['country'],
            'postalCode'  => $customerAddress['postcode'],
            'floor'       => "",
            'door'        => ""
        ];

        $senderPhoneFormated   = $this->PhoneNumberUtil->parse($this->config->getValueByKey('senderAddressPhoneNumber'), $senderAddress['CountryCode']);

        $comments = $order->get_customer_note();
        if (!$comments) {
            $comments = '';
        }

        $contact = $customerAddress['first_name'] . ' ' . $customerAddress['last_name'];

        $pickupDatetime = $order->get_meta('_shipquik_pickupdatetime');

        // Pickup object
        $shipment['pickup'] = [
            "name"      => $this->config->getValueByKey('senderAddressCompany'),
            "cif"       => $this->config->getValueByKey('senderAddressCif'),
            "phone"     => [
                "countryCallingCode" => (string) $senderPhoneFormated->getCountryCode(),
                "number"             => $senderPhoneFormated->getNationalNumber(),
                "numberIntl"         => '+' . $senderPhoneFormated->getCountryCode() . ' ' . $senderPhoneFormated->getNationalNumber()
            ],
            "email"     => $this->config->getValueByKey('senderAddressEmail'),
            "comments"  => $this->config->getValueByKey('comments'),
            "attention" => '',
            'date' => (isset($pickupDatetime['pickup_date']) ? $pickupDatetime['pickup_date'] : ''),
            'schedule' => (isset($pickupDatetime['pickup_time']) ? $pickupDatetime['pickup_time'] : '1')
        ];

        $name = ($customerAddress['company'] ? $customerAddress['company'] : $customerAddress['first_name'] . ' ' . $customerAddress['last_name']);
        $cif = '';

        $phone = [];

        if (isset($customerAddress['phone']) && $customerAddress['phone']) {
            $customerPhoneFormated = $this->PhoneNumberUtil->parse($customerAddress['phone'], $customerAddress['country']);

            $shipment['delivery'] = [
                "name"      => $name,
                "cif"       => $cif,
                "phone"     => [
                    "countryCallingCode" => (string) $customerPhoneFormated->getCountryCode(),
                    "number"             => $customerPhoneFormated->getNationalNumber(),
                    "numberIntl"         => '+' . $customerPhoneFormated->getCountryCode() . ' ' . $customerPhoneFormated->getNationalNumber()
                ],
                "email"     => $customerAddress['email'],
                "comments"  => $comments,
                "attention" => $contact,
            ];
        } else {
            $shipment['delivery'] = [
                "name"      => $name,
                "cif"       => $cif,
                'phone' => [
                    'countryCallingCode' => (string) $senderPhoneFormated->getCountryCode(),
                    'number' => $senderPhoneFormated->getNationalNumber(),
                    'numberIntl' => '+' . $senderPhoneFormated->getCountryCode() . ' ' . $senderPhoneFormated->getNationalNumber(),
                ],
                "email"     => $customerAddress['email'],
                "comments"  => $comments,
                "attention" => $contact,
            ];
        }

        foreach ($shippingMethod->get_meta_data() as $meta) {
            $item = $meta->get_data();
            if ($item['key'] == '_supplier_data') {
                $supplierInfo = json_decode(str_replace('&quot;', '"', $item['value']), true, 99);

                break;
            }
        }

        $supplierCall = new ShipQuik_Supplier($this->parent, $shipment['shipment']['boxes'], $customerAddress);

        $suppliers = [
            "name" => $supplierInfo['supplier'],
            "services" => [ $supplierInfo['service'] ]
        ];
        $supplierInfo = $supplierCall->send($suppliers, true);

        $supplierInfo = (isset($supplierInfo[0]) ? $supplierInfo[0] : []);

        if (empty($supplierInfo)) {
            return [];
        }

        // Supplier object
        $shipment["supplier"] = $supplierInfo;
        $shipment["supplier"]['currency'] = "";
        $shipment["supplier"]['serviceCode'] = (isset($supplierInfo['serviceCode']) ? $supplierInfo['serviceCode'] : '');

        $ivaPercent = 0;
        $clientType = $this->config->getClientType();

        if ($clientType == 0) {
            $resultTaxes = $this->getTaxesInfo($customerAddress, $senderAddress);

            if ($resultTaxes) {
                $ivaInfo = $resultTaxes['ivaInfo']['percentage'];
            }

            if ($resultTaxes) {
                $ivaInfo = $resultTaxes['ivaInfo']['percentage'];
                $shipment['iva'] = [
                    'ivaPercentage' => $resultTaxes['ivaInfo']['percentage'],
                    'ivaVentaName' => $resultTaxes['ivaInfo']['nameIva'],
                    'ivaCompraName' => 'IGIAND45',
                    'operationVName' => $resultTaxes['operationV'],
                    'operationCName' => $resultTaxes['operationC'],
                ];
            }
        } else {
            $shipment['iva'] = [
                'ivaPercentage' => 0,
                'ivaVentaName' => 'NOSUJETO',
                'ivaCompraName' => 'IGIAND45',
                'operationVName' => 'VNAC',
                'operationCName' => 'SUJPAS',
            ];
        }

        $duaImport = $supplierInfo['dua'];
        $insurance = 0;
        $insurance = 0;

        // INSURANCE object
        // pendiente del endpoint y usar Helper::getInsurance()
        $shipment['insurance'] = [
            "insurancePrice"   => 0,
            "declaredValue"    => 0,
            "insuranceCost"    => 0,
            "declaredObjects"  => "Documentos",
            "contentAgreement" => true,
            "isInternational"  => false,
            "isTaric"          => false,
            "tarics"           => []
        ];

        $shipment['payment'] = [
            "total"  => $supplierInfo['salesPrice'] + ($supplierInfo['salesPrice'] * ($ivaPercent / 100)) + $duaImport + $insurance,
            "sale"   => $supplierInfo['salesPrice'],
            "method" => (string) $this->config->getPaymentMethod()
        ];

        $uuid = ShipQuik_Helper::getUUID($supplierInfo['supplier']);

        $shipment["orderId"] = (string) $order->get_order_number();

        $shipment["reference"] = $uuid;
        $shipment["userEmail"] = $this->config->getValueByKey('senderAddressEmail');
        $shipment["clientId"] = $this->config->getIdCliente();

        if (!$order->get_meta('_shipquik_payment_status')) {
            $order->update_meta_data('_shipquik_payment_status', '0');
            $order->update_meta_data('_shipquik_payment', $shipment);
        }

        $order->save();

        return $shipment;
    }

    public static function getSenderAddress($config)
    {
        $senderAddress = [];

        $senderAddress['Street'] = $config->getValueByKey('senderAddressStreet');
        $senderAddress['City'] = $config->getValueByKey('senderAddressCity');
        $senderAddress['CountryCode'] = $config->getValueByKey('senderAddressCountryCode');
        $senderAddress['PostalCode'] = $config->getValueByKey('senderAddressPostalCode');

        $senderAddress['Country'] = WC()->countries->countries[$config->getValueByKey('senderAddressCountryCode')];

        return $senderAddress;
    }

    public function send()
    {
        $params = [];
        $params['Baseurl']  = $this->config->getValueByKey('endpoints')['createShipment'];
        $params['Data']     = $this->data;

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == "renew_token") {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        return $result;
    }

    public function getTaxesInfo($customerAddress = [], $senderAddress = [])
    {
        $taxesInfo = [
            "countryCodeOrigin"      => $senderAddress['CountryCode'],
            "countryCodeDestination" => $customerAddress['country'],
            "postalCodeOrigin"       => $senderAddress['PostalCode'],
            "postalCodeDestination"  => $customerAddress['postcode'],
            "idUser"                 => $this->config->getIdCliente()
        ];
    
        $shippingTaxes = new ShipQuik_ShippingTaxes($this->parent, $taxesInfo);
        $resultTaxes = $shippingTaxes->send();

        if ($resultTaxes) {
            return $resultTaxes;
        }

        return [];
    }

    public function getDuaInfo($boxes = [], $destination = [], $supplier = '', $declaredValue = 0)
    {
        $duaInfo= new ShipQuik_ShippingDua($this->parent, $boxes, $destination, $supplier, $declaredValue);
        $resultDuaInfo = $duaInfo->send();

        if ($resultDuaInfo) {
            return $resultDuaInfo;
        }

        return [];
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
