<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class ShipQuik_Curl
{
    private static $_instance = null;

    protected $_params = [];

    protected $parent = null;

    protected $config = null;

    protected $_is_post = true;

    protected $_verbose = false;

    public function __construct($parent = null, $params = [], $verbose = false, $is_post = true)
    {
        $this->parent  = $parent;
        $this->config   = $parent->config;

        $this->_params  = $params;
        $this->_is_post = $is_post;
        $this->_verbose = $verbose;
    }

    public function curlCall($connecttimeout = 800, $timeout = 800)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $this->_params['Baseurl']);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_NOSIGNAL, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $connecttimeout);
        curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
        //curl_setopt($curl, CURLOPT_HEADER, 1);

        $opciones = [
            'Content-Type: application/json; charset=utf-8',
            'Content-Length: ' . strlen(json_encode($this->_params['Data'])),
            'Expect: '
        ];

        $token = $this->config->getAccessToken();

        if ($token) {
            array_push($opciones, 'access-token: ' . $token);
        }

        if ($this->_is_post) {
            curl_setopt($curl, CURLOPT_POST, $this->_is_post);
            curl_setopt(
                $curl,
                CURLOPT_POSTFIELDS,
                json_encode($this->_params['Data'])
            );
            curl_setopt(
                $curl,
                CURLOPT_HTTPHEADER,
                $opciones
            );
        }

        if ($this->_verbose) {
            curl_setopt($curl, CURLOPT_VERBOSE, true);
        }

        $response = curl_exec($curl);
        $err = curl_error($curl);

        $response_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($response_status == 401) {
            curl_close($curl);

            return "renew_token";
        }

        curl_close($curl);

        if ($this->config->getValueByKey('debug') || $err) {
            $this->parent->logger->info(
                'CURL: Response: ' . wc_print_r(json_decode($response, true), true),
                array('source' => 'ship-quik')
            );
            $this->parent->logger->info(
                'CURL: Error: ' . wc_print_r($err, true),
                array('source' => 'ship-quik')
            );
        }

        if ($err) {
            return false;
        }
        $data = json_decode($response, true);

        return $data;
    }

    public static function instance()
    {
        if (! isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }
        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
