<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use ShipQuik\WooCommerce\ShipQuik_Shipments;

class ShipQuik_PickupDate
{
    private static $_instance = null;

    protected $parent;

    protected $mydate;

    protected $config;

    protected $data;

    public function __construct($parent = null, $mydate)
    {
        $this->parent = $parent;
        $this->mydate = $mydate;

        $this->config = $parent->config;

        $this->data = $this->getData($mydate);
    }

    private function getData()
    {
        $pickupDate = [];

        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $pickupDate['pickupDate'] = $this->mydate;

        $pickupDate['shipment']['countryCodeOrigin'] = $senderAddress['CountryCode'];
        $pickupDate['shipment']['originAddress']['city'] = $senderAddress['City'];
        $pickupDate['shipment']['originAddress']['postalCode'] = $senderAddress['PostalCode'];

        return $pickupDate;
    }

    public function send()
    {
        $result = false;

        $params = [];

        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['pickupDate'];
        $params['Data'] = $this->data;

        // d(json_encode($params['Data']));

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();
            $result = $curl->runCall();
        }

        return $result;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
