<?php
/**
Functions for Ship-Quik
 **/

namespace ShipQuik\WooCommerce;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use ShipQuik\Shared\ShipQuik_Configuration;

class ShipQuik_ShippingPackage
{
    protected $parent;

    protected $config;

    protected $package;

    protected $measureUnit;

    public function __construct($parent, $package, $measureUnit = 'kg')
    {
        $this->parent      = $parent;
        $this->config      = $parent->config;

        $this->package     = $package;
        $this->measureUnit = $measureUnit;
    }

    public function getDestination()
    {
        return $this->package['destination'];
    }

    public function getProducts()
    {
        return $this->package['contents'];
    }

    /**
     * Calculates the cubage of all products.
     *
     * @return float|int
     */
    public function getTotalCubage()
    {
        $total = 0;

        foreach ($this->package['contents'] as $item_id => $values) {
            $_product = $values['data'];
            $_product_height = (float) $_product->get_height();
            $_product_width  = (float) $_product->get_width();
            $_product_length = (float) $_product->get_length();

            if ($_product_height == 0) {
                $_product_height = $this->config->getValueByKey('defaultParcelHeight');
            }
            if ($_product_width == 0) {
                $_product_width = $this->config->getValueByKey('defaultParcelWidth');
            }
            if ($_product_length == 0) {
                $_product_length = $this->config->getValueByKey('defaultParcelLength');
            }

            $total += $_product_height * $_product_width * $_product_length;
        }

        return number_format(( ($total > 0) ? ( $total / 1000000 ) : 0 ), 2); // cubit meters
    }

    public static function getDefaultPackage($config)
    {
        $packagesInfo = $config->getValueByKey('shippingPackageList');

        foreach ($packagesInfo as $package) {
            if (isset($package['default'])) {
                return $package;
            }
        }

        return false;
    }

    /**
     * Get the boxes of all products.
     *
     * @return array
     */
    // public function getBoxes()
    // {
    //     $boxes = [];

    //     foreach ($this->package['contents'] as $item_id => $values) {
    //         $_product = $values['data'];

    //         $totalWeight = (string)((int) $values['quantity'] * ((float) $_product->get_weight() ? (float) $_product->get_weight() : $this->config->getValueByKey('defaultParcelWeight')));

    //         $boxes[] = [
    //             "weight" => $totalWeight,
    //             "length" => (string)((float) $_product->get_length() ? (float) $_product->get_length() : $this->config->getValueByKey('defaultParcelLength')),
    //             "width"  => (string)((float) $_product->get_width() ? (float) $_product->get_width() : $this->config->getValueByKey('defaultParcelWidth')),
    //             "height" => (string)((float) $_product->get_height() ? (float) $_product->get_height() : $this->config->getValueByKey('defaultParcelHeight')),
    //             "id"     => $i++
    //         ];
    //     }

    //     return $boxes;
    // }

    /**
     * Get the boxes of all products.
     *
     * @return array
     */
    public static function getBoxes($config, $productItems)
    {
        $boxes = [];

        $defaultPackage = self::getDefaultPackage($config);

        foreach ($productItems as $data) {
            $quantity = $data['quantity'];
            $_product = (isset($data['product_id']) ? wc_get_product($data['product_id']) : wc_get_product($data->get_product_id()));

            for ($i = 1; $i <= $quantity; ++$i) {
                if ((float) $_product->get_length() == 0 || (float) $_product->get_width() == 0 || (float) $_product->get_height() == 0 || (float) $_product->get_weight() == 0) {
                    $boxes[] = [
                        'weight' => (string) $defaultPackage['weight'],
                        'length' => (string) (float) $defaultPackage['depth'],
                        'width' => (string) (float) $defaultPackage['width'],
                        'height' => (string) (float) $defaultPackage['height']
                    ];
                } else {
                    $boxes[] = [
                        'weight' => (string) $_product->get_weight(),
                        'length' => (string) (float) $_product->get_length(),
                        'width' => (string) (float) $_product->get_width() ,
                        'height' => (string) (float) $_product->get_height()
                    ];
                }
            }
        }

        return $boxes;
    }

    /**
     * Calculate the total volumes of all products of the order
     *
     * @return int
     */
    public function getTotalVolumes()
    {
        $total = 0;

        foreach ($this->package['contents'] as $item_id => $values) {
            $total += (int) $values['quantity'];
        }

        return $total;
    }

    /**
     * Calculate the total wheight of all products of the order
     *
     * @return float|int
     */
    // public function getOrderTotalWeight()
    // {
    //     $total = 0;

    //     foreach ($this->package['contents'] as $item_id => $values) {
    //         $_product = $values['data'];
    //         $_product_weight = (float) $_product->get_weight();

    //         if ($_product_weight == 0) {
    //             $_product_weight = $this->config->getValueByKey('defaultParcelWeight');
    //         }

    //         $total += $_product_weight * $values['quantity'];
    //     }

    //     $total = wc_get_weight($total, $this->measureUnit);

    //     return $total;
    // }
}
